clear;



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%This program solves the gift-bequest problem for 2 states and various Pi's
%%%%%Utility is separable in consumption and effort, there is NO aggregate uncertainty.
%%%%%It uses files: pictures.m, system.m, w.m, w_b.m
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

global sigma gamma z_H z_L A u up v vp W W_b N Pivector Pi

%parameters
sigma = 2; %risk aversion in consumption
gamma = 2; %labor elasticity or something like that
z_H = 2;
z_L = 1;
A = 2;

if sigma == 1
   u  = @(c) log(c);
   up = @(c) 1./c;
else
    u  = @(c) c.^(1-sigma)./(1-sigma);
    up = @(c) c.^-sigma;
end

v  = @(l) l.^(1+gamma)./(1+gamma);
vp = @(l) l.^gamma;

%if z<>z_L and z<>z_H, takes it as z = z_L
W = @(b, z, Pi) (Pi*w(b,z_L) + (1-Pi)*w(b,z_H))*(z==z_L) + (Pi*w(b,z_H) + (1-Pi)*w(b,z_L))*(1-(z==z_L));
W_b = @(b, z, Pi) (Pi*w_b(b,z_L) + (1-Pi)*w_b(b,z_H))*(z==z_L) + (Pi*w_b(b,z_H) + (1-Pi)*w_b(b,z_L))*(1-(z==z_L));

N = 101; %number of values for Pi
Pivector = linspace(.5,1,N); %all the Pi's, x axis for pictures
policies = zeros(8,N);

%initial guess for Pi = 0.5
x0 = ones(8,1);
IC_L = zeros(1,N);
  tic;
for i = 1:N
  
    Pi = Pivector(i);
	options = optimset('display', 'off', 'tolfun', 10^(-10), 'maxfunevals', 1000000, 'maxiter', 1000000); 
    x = fsolve(@system, x0, options);
    %sum(abs(system(x)));
    
    %initial guess for next round
    x0 = x;
    policies(:,i) = x;
    IC_L(i) = u(x(1))-v(x(3)/z_L) + W(x(5), z_L, Pi) - u(x(2)) + v(x(4)/z_L) - W(x(6), z_L, Pi);
    i;
 
end
   toc;
%maybe should first compute the continuation value
%tic;
%vf = zeros(100,1);
%for i = 1:100
%    vf = W(i/100, z_H, .75);
%end
%toc;
z = policies;

c_L = z(1,:);
c_H = z(2,:);
y_L = z(3,:); 
y_H = z(4,:); 
b_L = z(5,:); 
b_H = z(6,:); 
lambda = z(7,:); 
mu = z(8,:);

min(IC_L);

g_L = c_L - y_L;
g_H = c_H - y_H;

output = [Pivector; policies(1:6,:); g_L; g_H; policies(7:8,:)];

pictures(policies);
