clear;
clc;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%This program solves the gift-bequest problem for 2 states and various
%%%%%diagonal Pi's and thetas
%%%%%Parents receive a signal about kids' productivity theta.
%%%%%Utility is separable in consumption and effort, there is NO aggregate uncertainty.
%%%%%It uses files: systems.m, systemeq.m, w.m, w_b.m
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

global sigma gamma z_H z_L A u up v vp W W_b N Pivector Pi theta

%parameters
sigma = 2; %risk aversion in consumption
gamma = 2; %labor elasticity or something like that
z_H = 2;
z_L = 1;
A = 2;

if sigma == 1
   u  = @(c) log(c);
   up = @(c) 1./c;
else
    u  = @(c) c.^(1-sigma)./(1-sigma);
    up = @(c) c.^-sigma;
end

v  = @(l) l.^(1+gamma)./(1+gamma);
vp = @(l) l.^gamma;

%if z<>z_L and z<>z_H, takes it as z = z_L
W = @(b, z, Pi) (Pi.*w(b,z_L) + (1-Pi).*w(b,z_H))*(z==z_L) + (Pi.*w(b,z_H) + (1-Pi).*w(b,z_L))*(1-(z==z_L));
W_b = @(b, z, Pi) (Pi.*w_b(b,z_L) + (1-Pi).*w_b(b,z_H))*(z==z_L) + (Pi.*w_b(b,z_H) + (1-Pi).*w_b(b,z_L))*(1-(z==z_L));

N = 21; %number of values for Pi
M = 21; %number of values for theta
Pivector = linspace(.5,1,N); 
thetavector = [linspace(.5, 1-.5/(M-1), M-1),.9999];
%thetavector = linspace(.5, 1, M);

%policies = zeros(15,N);
%load policies
%initial guess
x0 = ones(15,1);
xeq0 = ones(12,1);

%IC_L = zeros(1,N);
  tic;
for i = 1:N
    for j = 1:M
    %initial guess
    %x0 = policies(:, i, j);
    
    Pi = Pivector(i);
   
    theta = thetavector(j);
    options = optimset('display', 'off', 'tolfun', 10^(-10), 'maxfunevals', 1000000, 'maxiter', 1000000); 
   
    xeq = fsolve(@systemeq, xeq0, options); 
    x = fsolve(@systems, x0, options);
    %sum(abs(system(x)));
    
    %initial guess for next round
    x0 = x;
    xeq0 = xeq;
    policies(:,i, j) = x;
    policieseq(:, i, j) = xeq;
    
    %IC_L(i, j) = u(x(1))-v(x(3)/z_L) + W(x(5), z_L, Pi1) - u(x(2)) + v(x(4)/z_L) - W(x(6), z_L, Pi1);
    
    i;
    j;
    end
end
   toc;
%save policies policies
z = policies;
zeq = policieseq;


c_HL = squeeze(z(1,:, :));
c_HH = squeeze(z(2,:, :));
y_HL = squeeze(z(3,:,:)); 
y_HH = squeeze(z(4,:, :)); 
b_HL = squeeze(z(5,:, :)); 
b_HH = squeeze(z(6,:, :)); 
c_LL = squeeze(z(7,:, :));
c_LH = squeeze(z(8,:, :));
y_LL = squeeze(z(9,:,:)); 
y_LH = squeeze(z(10,:, :)); 
b_LL = squeeze(z(11,:, :)); 
b_LH = squeeze(z(12,:, :)); 
lambda = squeeze(z(13,:, :)); 
mu_HH= squeeze(z(14,:, :));
mu_LH= squeeze(z(15,:, :));
g_HH = c_HH - y_HH;
g_HL = c_HL - y_HL;
g_LH = c_LH - y_LH;
g_LL = c_LL - y_LL;

%generate theta and Pi matrix
thetamat = kron(thetavector, ones(N,1));
Pimat = kron(Pivector', ones(1,M));

V = thetamat.*(u(c_HH) + W(b_HH, z_H, Pimat) - v(y_HH/z_H)) + (1-thetamat).*(u(c_HL) + W(b_HL, z_L, Pimat) - v(y_HL/z_L)) + thetamat.*(u(c_LL) + W(b_LL, z_L, Pimat) - v(y_LL/z_L)) + (1-thetamat).*(u(c_LH) + W(b_LH, z_H, Pimat) - v(y_LH/z_H)) ;
%checking the BC
%thetamat.*(c_HH - y_HH + b_HH) + (1-thetamat).*(c_HL - y_HL + b_HL) + (1-thetamat).*(c_LH - y_LH + b_LH) + thetamat.*(c_LL - y_LL + b_LL) - A

c_HLeq = squeeze(zeq(1,:, :));
c_HHeq = squeeze(zeq(2,:, :));
y_HLeq = squeeze(zeq(3,:,:)); 
y_HHeq = squeeze(zeq(4,:, :)); 
c_LLeq = squeeze(zeq(5,:, :));
c_LHeq = squeeze(zeq(6,:, :));
y_LLeq = squeeze(zeq(7,:,:)); 
y_LHeq = squeeze(zeq(8,:, :)); 
beq = squeeze(zeq(9,:, :)); 
lambdaeq = squeeze(zeq(10,:, :)); 
mu_HHeq = squeeze(zeq(11,:, :));
mu_LHeq = squeeze(zeq(12,:, :));
g_HHeq = c_HHeq - y_HHeq;
g_HLeq = c_HLeq - y_HLeq;
g_LHeq = c_LHeq - y_LHeq;
g_LLeq = c_LLeq - y_LLeq;
Veq = thetamat.*(u(c_HHeq) + W(beq, z_H, Pimat) - v(y_HHeq/z_H)) + (1-thetamat).*(u(c_HLeq) + W(beq, z_L, Pimat) - v(y_HLeq/z_L)) + thetamat.*(u(c_LLeq) + W(beq, z_L, Pimat) - v(y_LLeq/z_L)) + (1-thetamat).*(u(c_LHeq) + W(beq, z_H, Pimat) - v(y_LHeq/z_H)) ;
%checking the BC
%thetamat.*(c_HHeq - y_HHeq) + (1-thetamat).*(c_HLeq - y_HLeq) + (1-thetamat).*(c_LHeq - y_LHeq) + thetamat.*(c_LLeq - y_LLeq) + 2*beq - A
 
kappa = (V - Veq)./lambdaeq;

%computing the variance/std of bequests accross types
meanb = (thetamat.*b_HH + (1-thetamat).*b_HL + (1-thetamat).*b_LH + thetamat.*b_LL)/2;
varb = (thetamat.*(b_HH-meanb).^2 + (1-thetamat).*(b_HL-meanb).^2 + (1-thetamat).*(b_LH-meanb).^2 + thetamat.*(b_LL-meanb).^2 )/2;
stdb = varb.^.5;
diffb_H = b_HH - b_LH;

% figure(1)
% mesh(thetavector, Pivector, varb)
% ylabel('symmetric \pi')
% xlabel('\theta')
% zlabel('var_b')
% title('Variance of Bequests')
% 
% figure(2)
% mesh(thetavector, Pivector, stdb)
% ylabel('symmetric \pi')
% xlabel('\theta')
% zlabel('std_b')
% title('Standard Deviation of Bequests')
% 
% figure(3)
% mesh(thetavector, Pivector, kappa)
% ylabel('symmetric \pi')
% xlabel('\theta')
% zlabel('\kappa')
% title('Kappa')

indplots = 0;
if indplots == 1
for i = 1:N
    figure(i)
    plot(thetavector, kappa(i,:))
    xlabel('\theta')
    ylabel('\kappa')
    title('Kappa')
end

figure(N+1)
mesh(thetavector, Pivector, kappa)
ylabel('symmetric \pi')
xlabel('\theta')
zlabel('\kappa')
title('Kappa')
end

figure(1)
mesh(thetavector, Pivector, Veq)
%hold on
%mesh(thetavector, Pivector, V)
xlabel('\theta')
ylabel('symmetric \pi')
zlabel('V')
title('Value functions')

figure(2)
mesh(thetavector, Pivector, lambdaeq)
%hold on
%mesh(thetavector, Pivector, lambda)
xlabel('\theta')
ylabel('symmetric \pi')
zlabel('dV/dA')
title('Value function derivatives')

