%alternative transition matrix routine using error funcions
%here each z_p implies an equally spaced vector of z_1
function TMR6 = TMR6alt(x)
global N mu_A sigma_A N2


%it solves for the grid points and bin endpoints given the input
%parameters

%Parents' skills and assets assumed jointly log-normal
%(log A, log z_P) ~ N((mu_A, mu_P), [sigma_A^2, rho_zPA sigma_A sigma_zP;   rho_zPA sigma_A sigma_zP, sigma_zP^2])

%Process for kids' first period's skills
%log z_1 = rho_zG (log z_P - mu_zP) + e_zG + mu_z1
%e_G ~ N(0, sigma_zG^2)
%note: log z_1 ~ N(mu_z1, rho_zG^2*sigma_zP^2 + sigma_zG^2)
%note: log z_1|log z_P ~ N(rho_zG*(log z_P - mu_zP) + mu_z1, sigma_zG^2)


%Process for kids' second period's skills
%log z_2 = rho_L (log z_1 - mu_z1) + e_zL + mu_z2
%e_L ~ N(0, sigma_zL^2)
%note: log z_2 ~ N(mu_z2, rho_zL^2*(rho_zG^2* sigma_zP^2 + sigma_zG^2) + sigma_zL^2)
%note: log z_2|log z_1 ~ N(rho_zL*(log z_1 - mu_1) + mu_z2, sigma_zL^2)

%Define values of parameters (could make it a function with these inputs later)
mu_zP    = x(1);
rho_zPA  = x(2);
mu_z1    = x(3);
sigma_zG = x(4);
rho_zG   = x(5);
mu_z2    = x(6); 
sigma_zL = x(7);
rho_zL   = x(8);

alpha = 1;

%just use the equations and SS assumption
%sigma_zP = ((rho_zL^2*sigma_zG^2 + sigma_zL^2)/(alpha-rho_zG^2*rho_zL^2))^.5;
sigma_zP = 0.78;
%load sigma_zP
%sigma_zP
%sigma_zL = .6
%sigma_zP = 1.2007
%sigma_z1 = ((rho_zG^2*sigma_zL^2 + alpha*sigma_zG^2)/(alpha-rho_zG^2*rho_zL^2))^.5;
%sigma_z2 = alpha*sigma_zP; %not used anyweher but in the above calculations

%Define the number of grid points
n = N;


%gridpoints and bins on log z_P
grid_P = erfinv(2*( 1/(2*n):1/n:1-1/(2*n))-1)*2^.5*sigma_zP+ mu_zP; %this is s.d.
bin_P = erfinv(2*( 1/n:1/n:1)-1)*2^.5*sigma_zP+ mu_zP;
%gridpoints and bins on log z_1, here we could integrate over bin_zP to get
%a better approximation
grid_1 = erfinv(2*( kron(1/(2*n):1/n:1-1/(2*n),ones(n,1)))-1)*2^.5.*(sigma_zG*ones(n,n))+ kron(mu_z1 + rho_zG*(grid_P' - mu_zP),ones(1,n));
bin_1 = erfinv(2*( kron(1/n:1/n:1,ones(n,1)))-1)*2^.5.*(sigma_zG*ones(n,n))+ kron(mu_z1 + rho_zG*(grid_P' - mu_zP),ones(1,n));

%bin_1 = icdf('normal', 1/n:1/n:1, mu_z1, sigma_z1);

%gridpoints and bins on log z_2, evenly spaced on z_2|z_P ~ N(mu_z2 + rho_zL *rho_zG*(z_P - mu_zP), rho_zL^2*sigma_zG^2 + sigma_zL^2)
grid_2 = erfinv(2*(kron(1/(2*N2):1/N2:1-1/(2*N2),ones(N,1)))-1)*2^.5.*(((rho_zL^2*sigma_zG^2 + sigma_zL^2))^.5*ones(N, N2))+kron(mu_z2 + rho_zL *rho_zG*(grid_P' - mu_zP), ones(1,N2));
bin_2 = erfinv(2*(kron(1/N2:1/N2:1,ones(N,1)))-1)*2^.5.*(((rho_zL^2*sigma_zG^2 + sigma_zL^2))^.5*ones(N, N2))+kron(mu_z2 + rho_zL *rho_zG*(grid_P' - mu_zP), ones(1,N2));

%here I need to integrate
%transition matrix P(z_2|z_1)

    k = 10000; %number of gridpoints for numerical integration, no change in results if 
    %we go from 1000 to 10000 can as well leave 10000, takes no time
    
    %for each n  conditional distributions of z_1|z_P
    grid_12 = erfinv(2*( kron(1/(2*k):1/k:1-1/(2*k),ones(n,1)))-1)*2^.5.*(sigma_zG*ones(n,k))+ kron(mu_z1 + rho_zG*(grid_P' - mu_zP),ones(1,k));

    
        for j = 1:n
                   
            %transition probabilities for each point on the big grid into z_2
             (kron(ones(k,1),bin_2(j,:)));
             rho_zL*(grid_12(j,:)'*ones(1,N2) - mu_z1) + mu_z2;
             
            matrix = 1/2*(1+ erf((kron(ones(k,1),bin_2(j,:)) - (rho_zL*(grid_12(j,:)'*ones(1,N2) - mu_z1) + mu_z2))./(2^.5*sigma_zL*ones(k,N2)) ));
            
            %index = round(cdf('normal', bin_1(j,:), mu_z1, sigma_z1)*k);
            TL = zeros(n, N2);
            for i = 1:n
                TL(i,:) =  sum(matrix((i-1)*k/n+1:k/n*i,:))/(k/n);
                %if i == 1
                %   TL(i,:)  =  sum(matrix(1:index(1),:))/index(1);
                %else
                %   TL(i,:) = sum(matrix(index(i-1)+1: index(i),:))/(index(i)-index(i-1));
                %end
            
                
            end
   
            TL(:, 2:N2) = TL(:, 2:N2) - TL(:, 1:N2-1); 
            output((j-1)*n+1: j*n,:) = TL;
        end
    
    %output n^2 x n^2 matrix
    TMR6 = [output;grid_2];
    
