*Please, use Stata 13.

clear all
set more off
cd "c:\Users\cslavik\Research\AAA Optimal Contracting\Bequests\our paper\RED submsission\Codes and Data\PSID Data and Codes\"

use Data_panel_family.dta, clear

*THIS IF FOR THE INTRAGENERATIONAL PERSISTENCE*
*it uses WLS, weighting observations by the inverse of the number of observations for a given agent

*************************************************
* 1. tracked years and interviews
*************************************************
sort identifier year

********
*CHECK IF WE DO ONLY MEN
tab SexHead
keep if SexHead == 1
********

replace Education = . if Education == 0 | Education == 99

bys identifier: egen TrackIntwsHead = count(_n)
bys identifier: gen  TrackIntwsHeadOne = TrackIntwsHead if _n == 1
label variable TrackIntwsHead "Intws track the Head"
label variable TrackIntwsHeadOne "Intws track the Head one entry"

bys identifier: egen maxYear = max(year)
bys identifier: egen minYear = min(year)
bys identifier: gen TrackYearsHead = maxYear - minYear
bys identifier: replace TrackYearsHead = 1 if TrackYearsHead == 0
bys identifier: gen TrackYearsHeadOne = TrackYearsHead if _n == 1
label variable TrackYearsHead "Years track the Head"
label variable TrackYearsHeadOne "Years track the Head one entry"
drop maxYear minYear

************************************************
* 1b. HPV sampling
* A. HPV procedure drop either the head or spouse has positive labor income but zero annual hours
************************************************
drop if HeadTotalWage>0 & HeadHours==0

*drop if HeadTotalWage == . & HeadHours > 0
*will not be used anyways

*additional cleaning
drop if HeadTotalWage==0 & HeadHours>0

*drop if HeadTotalWage == . & HeadHours > 0
*will not be used anyways

drop if HeadWage_gen>0 & HeadWage_gen<.1*MinWageNom

*some checking
order identifier year HeadWageHour HeadWage_gen  HeadHours HeadTotalWage HeadWage

*additional cleaning
replace HeadWage_gen=0 if HeadHours==0 & HeadTotalWage==0

*************************************************
* 2. real wages
* note that in the data I use interview year, which is reporting for last year's info
* I adjusted the year cpi by -1
*************************************************

gen HeadWageHourReal = HeadWage_gen/cpi*100
*gen WifeWageHourReal = WifeWageHour/cpi*10000

*************************************************
* 3. wage correlations t and t-25, in logs
*************************************************
xtset identifier year
*gen logHeadWageHourReal = log(HeadWageHourReal)

*extrapolating for missing years
xtset identifier year
expand 2 if year> 1997
sort identifier year
quietly by identifier year:  gen dup = cond(_N==1,0,_n)
replace year = year-1 if year> 1997 & dup==2
xtset identifier year
replace HeadWageHourReal = (l1.HeadWageHourReal + f1.HeadWageHourReal)/2 if year> 1997 & dup==2
replace AgeHead = (l1.AgeHead + f1.AgeHead)/2 if year> 1997 & dup==2
drop if HeadWageHourReal ==. & year> 1997 & dup==2
*need both values around

*fixing inconsistencies in age
*gen issue = 1 if AgeHead == l1.AgeHead
bys identifier: egen age_min = min(AgeHead)
bys identifier: egen year_min = min(year)
gen age = age_min + year - year_min 
replace AgeHead = age
*I think of ages as 25.00 - 49.99 and 50.00 - 74.99
drop if AgeHead < 25
drop if AgeHead > 74

gen meanwage = HeadWageHourReal
gen logmeanwage = log(HeadWageHourReal)
gen meanwageold = meanwage

xtset identifier year

gen L25logmeanwage = 0
gen number = 1
gen count2 = .
gen weight = .
gen aw = .
*gen laglogmeanwage = 0

*generate average wage when young and old
forvalues tt = 1(1)12 {
*display "Aggregation level " `tt'
replace logmeanwage = log(meanwage)
replace L25logmeanwage = L25.logmeanwage

*generating weights
drop count2 weight aw
gen count2 = 1 if logmeanwage~=. & L25logmeanwage~=.
by identifier, sort: egen weight = sum(count2) 
gen aw = count2/weight
qui replace meanwageold = meanwage*number
replace number = number+1
replace meanwage = (meanwageold+f`tt'.HeadWageHourReal)/number
}

sum logmeanwage [aw=aw] if logmeanwage!=. & L25logmeanwage!=.
sum L25logmeanwage [aw=aw] if logmeanwage!=. & L25logmeanwage!=. 
regress logmeanwage L25logmeanwage [aw=aw]
