%% Load the data, if you updated the excel file KeyCode.xlsx 
close all; clear all; clc;
[~, ~, KeyCode] = xlsread('/Users/Jun/Work/Ctirad/PSID/Bequests/Data/KeyCode.xlsx','Sheet1');
KeyCode = KeyCode(1:end,:);
KeyCode(cellfun(@(x) ~isempty(x) && isnumeric(x) && isnan(x),KeyCode)) = {''};

% get the variable names and the data
data = KeyCode(2:end,:);
name = KeyCode(1,:);
emptyIndex = cellfun(@isempty,data);       %# Find indices of empty cells
data(emptyIndex) = {'XX,9X9X9X'};           % Just set it as an indicator, since 9x9x9x never shows up

nYears = size(data, 1);
nNames = size(name, 2);
% fill the empty cells with XX,9X9X9X
% no year equal to XX, no variable name starts with 9
for i = 1:nYears
   if strcmp(data{i, 1},'XX,9X9X9X') && strcmp(data{i, 2},'XX,9X9X9X') && strcmp(data{i, 3},'XX,9X9X9X')
       data(i, :) = [];
   end
end
nYears = size(data, 1);

% generate the variables
for j = 1:nNames
    eval([name{j} '= data(:,j);']);
end

display('Variables included:');
display(name');


%% separate the variable and the corresponding years

data_year = data; % construct year data

for i = 1:nYears
    for j = 1:nNames
        temp = strsplit(data{i,j}, ',');
        data_year{i, j} = temp{1};
%         temp_name =strcat(name{j}, 'Year{i}');
        data{i, j} = temp{2};
        
%         temp = eval('strsplit(',name{j},'), ','');
%         eval(['temp= strsplit(',name{j},'), ','']);
        
    end
end

% change the year from two digits to 4 digits

for i = 1:nYears
    for j = 1:nNames
        if ~strcmp(data_year{i, j},'xx')
            if str2num(data_year{i, j}) > 50
                data_year{i, j} = strcat('19', data_year{i, j});
            elseif str2num(data_year{i, j}) < 50
                data_year{i, j} = strcat('20', data_year{i, j});
            end
            
        end
        
    end
end


%%

fileID = fopen('PSIDrename.do', 'w');
fprintf(fileID, '***********************************\n*The STATA code is generated automatically\n*****************************\n');
fprintf(fileID, '* Created by Jun Li\n');
fprintf(fileID, '* junli.econ@gmail.com\n');
fprintf(fileID, '***********************************\n');
fprintf(fileID, 'set more off \nclear all\n');
fprintf(fileID, 'cd ~/Work/Ctirad/PSID/Bequests/Data\n');
fprintf(fileID, 'use Data_raw_current.dta, clear \n');
fprintf(fileID, 'gen identifier = 1000*ER30001 + ER30002\n');
fprintf(fileID, 'gen FatherPN = ER32017\n');
fprintf(fileID, 'gen MotherPN = ER32010\n');

for i = 1:nYears
    for j = 1:nNames
        if ~strcmp(data{i, j},'9X9X9X')
        fprintf(fileID, 'rename %s %s%s\n', data{i, j}, name{j}, data_year{i, j});
        end
    end     
    fprintf(fileID, '\n');
end


fprintf(fileID, 'drop ER*\n');
fprintf(fileID, 'drop V*\n');
fprintf(fileID, 'gen ID1968= IW1968\n');

fprintf(fileID, '* get the unique identifier by ID1968*1000+ personal number\n');

fprintf(fileID, 'reshape long ');
for j = 1:nNames
    fprintf(fileID, '%s ', name{j});
end
fprintf(fileID, ', i(identifier) j(year)\n');
% fprintf(fileID, 'drop if AgeHead == .\n'); 
fprintf(fileID, 'sort ID year SequenceNum Relation\n');
fprintf(fileID, 'order ID year SequenceNum Relation\n');
fprintf(fileID, 'save Data_reshaped.dta, replace\n');

fclose(fileID);
display('Finished Check file PSIDrename.do');