*Please, use Stata 13.

clear all
set more off
set type double, permanently
cd "c:\Users\cslavik\Research\AAA Optimal Contracting\Bequests\our paper\RED submsission\Codes and Data\PSID Data and Codes\"

use Data_panel_ID.dta, clear
*************************************************
* 1. tracked years and interviews
*************************************************
sort identifier year
*order identifier year ID68 PN68 AgeHead relation

* SRC sample
keep if ID<= 3000


generate ID68 = int(identifier/1000)
label variable ID68 "Identification number in the 1968 survey"
generate PN68 = identifier-ID68*1000
label variable PN68 "Person number in the 1968 survey"
drop ID

*focus on fathers and sons as in the literature
keep if SexHead == 1

*keeping track only of biological children
replace Head = 3 if Relation == 3 & year >= 1968 & year <= 1982
replace Head = 3 if Relation == 30 & year >= 1983

order identifier year ID68 PN68 AgeHead AgeWife Relation Head SexHead IW FatherPN MotherPN

xtset identifier year

keep if Head == 1

************************************************
* 1b. HPV sampling
* A. HPV procedure drop either the head or spouse has positive labor income but zero annual hours
************************************************
drop if HeadTotalWage>0 & HeadHours==0

*additional cleaning
drop if HeadTotalWage==0 & HeadHours>0

*drop if HeadTotalWage == . & HeadHours > 0
*will not be used anyways

drop if HeadWage_gen>0 & HeadWage_gen<0.1*MinWageNom

*some checking
order identifier year HeadWageHour HeadWage_gen  HeadHours HeadTotalWage HeadWage

*additional cleaning
replace HeadWage_gen=0 if HeadHours==0 & HeadTotalWage==0

*************************************************
* TOP CODING
*************************************************
*there is a very small numer of top-coded observations, so follow HSV and do nothing,


*************************************************
* 2. real wages and wealth
* note that in the data I use interview year, which is reporting for last year's info
* I adjusted the year cpi by -1
*************************************************

gen HeadWageHourReal = HeadWage_gen/cpi*100
*gen WifeWageHourReal = WifeWageHour/cpi*10000

gen WealthReal = WealthInclH/cpi*100
gen EarningsReal =  HHWage/cpi*100
gen EarningsHeadReal = HeadTotalWage/cpi*100


keep identifier year ID68 PN68 AgeHead HeadWageHourReal WealthReal EarningsReal EarningsHeadReal 

*extrapolating wages for missing years 1998, 2000, 2002 etc.
xtset identifier year
expand 2 if year> 1997


sort identifier year
quietly by identifier year:  gen dup = cond(_N==1,0,_n)
replace year = year-1 if year> 1997 & dup==2
xtset identifier year
replace HeadWageHourReal = (l1.HeadWageHourReal + f1.HeadWageHourReal)/2 if year> 1997 & dup==2
replace WealthReal = (l1.WealthReal + f1.WealthReal)/2 if year> 1997 & dup==2
replace EarningsReal = (l1.EarningsReal + f1.EarningsReal)/2 if year> 1997 & dup==2
replace EarningsHeadReal = (l1.EarningsHeadReal + f1.EarningsHeadReal)/2 if year> 1997 & dup==2
*Age will be recalculated again later anyways
replace AgeHead = (l1.AgeHead + f1.AgeHead)/2 if year> 1997 & dup==2
drop if HeadWageHourReal ==. & year> 1997 & dup==2


gen issue = 1 if AgeHead == l1.AgeHead
bys identifier: egen age_min = min(AgeHead)
bys identifier: egen year_min = min(year)
gen age = age_min + year - year_min 

*I think of ages as 25.00 - 49.99 and 50.00 - 74.99
drop if age < 25
drop if age > 74


drop AgeHead issue age_min year_min dup
save DataWealth.dta, replace

**************************************************************************************************
*wealth is not subject to top and bottom coding

use DataWealth.dta, clear
generate logwealth = log(WealthReal)

*************************************************
* wealth mean and variance for parents
*************************************************

*foreach tt in 1984 1989 1994 1999 2001 2003 2005 2007 2009 2011 2013 {
*sum logwealth if age == 50 & year == `tt'
*}
sum logwealth if age == 50 & (year==1984|year==1989|year==1994|year==1999|year==2001|year==2003|year==2005|year==2007|year==2009|year==2011|year==2013)
*sum logwealth if age == 50
 
*************************************************
* wage - wealth correlations for parents
*************************************************
generate meanwage = HeadWageHourReal
generate logmeanwage = log(HeadWageHourReal)

/*sum logmeanwage logwealth if age == 50
sum logmeanwage if age == 50
putexcel  B25=(r(N)) C25=(r(mean)) D25=(r(sd)) using "Results.xlsx", sheet("WageWealth2") modify
corr logmeanwage logwealth if age == 50
putexcel  F25=(r(N)) G25=(r(rho)) using "Results.xlsx", sheet("WageWealth2") modify
*/

gen meanwageold = meanwage 
gen number = 1
*generate average wage of parents (50-75)
foreach tt in 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 {
*display "Aggregation level " `tt'+1
qui replace number = number+1
qui replace meanwage = (meanwageold+f`tt'.HeadWageHourReal)/number
qui replace logmeanwage = log(meanwage)
*qui sum logmeanwage if age == 50
*putexcel  B`tt'=(r(N)) C`tt'=(r(mean)) D`tt'=(r(sd)) using "Results.xlsx", sheet("WageWealth2") modify

*sum logwealth if age == 50 & logmeanwage ~=. & logwealth ~=.

*qui corr logmeanwage logwealth if age == 50
*putexcel  F`tt'=(r(N)) G`tt'=(r(rho)) using "Results.xlsx", sheet("WageWealth2") modify
*regress logmeanwage logwealth if age == 50

qui replace meanwageold = meanwage*number
}

corr logmeanwage logwealth if age == 50

*************************************************
* savings function for kids
*************************************************
gen earningsold = EarningsReal
*replace number = 1
*generate average earnings of kids (25-50)
foreach tt in 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 {
qui replace earnings = earningsold+l`tt'.EarningsReal
qui replace earningsold = earnings
}

qui replace earnings = earningsold
qui gen logear = log(earnings)
regress logwealth logear if age == 50
*regress logwealth logear if age == 49
*sum logwealth logear if age == 50 & logear ~=. & logwealth ~=.
*sum logwealth logear if age == 49 & logear ~=. & logwealth ~=.
*graph twoway (scatter logwealth logear , msymbol(+)) if age == 50
*graph twoway (scatter logwealth logear , msymbol(+)) if age == 49
